﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Calculadora
{

    /// <summary>
    /// Formulario principal de la calculadora
    /// </summary>
    public partial class Principal : Form
    {

        string numeroAnterior = "";
        string ultimaOperacion = "";

        /// <summary>
        /// Constructor del formulario (Inicializa los controles y asigna los capturadores de eventos).
        /// </summary>
        public Principal()
        {
            InitializeComponent();

            foreach (Control item in this.Controls)
            {
                if (item is Button & ValidarSiEsNumero(item.Text))
                {
                    item.Click += DigitoClick;
                }
            }

            BotonSuma.Click += OperacionBinariaClick;
            BotonResta.Click += OperacionBinariaClick;
            BotonMultiplicacion.Click += OperacionBinariaClick;
            BotonDivision.Click += OperacionBinariaClick;

            BotonRaiz.Click += OperacionUnitariaClick;
        }

        /// <summary>
        /// Método que indica si el "String" indicado puede ser convertido a un numero.
        /// </summary>
        /// <param name="texto">String que se desea validar.</param>
        /// <returns>Retorna true si el texto puede convertirse a un numero entero o false en caso contrario.</returns>
        private bool ValidarSiEsNumero(string texto)
        {
            if (texto == null)
            {
                return false;
            }
            int numero = 0;
            if (int.TryParse(texto, out numero))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Método que obtiene una "Funcion" que se puede ejecutar tomando como identificador el "Signo"
        /// </summary>
        /// <param name="signo">Signo matemático que identifica a la operación</param>
        /// <returns>Retorna la funcion que se debe ejecutar para realizar la operacion deseada.</returns>
        private Func<double, double, double> ObtenerOperacion(string signo)
        {
            switch (signo)
            {
                case "+" : return (x, y) => x + y;
                case "-" : return (x, y) => x - y;
                case "*" : return (x, y) => x * y;
                case "/" : return (x, y) => x / y;
                case "%" : return (x, y) => (x * y) / 100 ;
                case "±" : return (x, y) => y * -1;
                case "S" : return (x, y) => Math.Sqrt(y);
                default : return (x, y) => y;
            }
        }

        /// <summary>
        /// Método que es ejecutado cada vez que se presiona un boton cuya operacion requiere de dos numeros.
        /// Ejemplo: suma, resta, multiplicacion y división.
        /// </summary>
        /// <param name="sender">Boton que ha presionado el usuario.</param>
        /// <param name="e">Argumentos del evento.</param>
        /// <remarks>
        /// Este metodo "emula" un stack almacenando el valor que el usuario haya introducido en 
        /// el cuadro de texto en la variable "numeroAnterior" para que despues este valor sea
        /// utilizado por la operacion binaria.
        /// </remarks>
        private void OperacionBinariaClick(object sender, EventArgs e)
        {
            var boton = sender as Button;
            numeroAnterior = textBox1.Text;
            ultimaOperacion = boton.Text;
            textBox1.Text = "";
        }

        /// <summary>
        /// Método que se ejecuta cuando el usuario presiona los botones que deben mostrar un resultado inmediato.
        /// Ejemplo: porcentaje, Raiz cuadrada.
        /// </summary>
        /// <param name="sender">Boton que ha presionado el usuario.</param>
        /// <param name="e">Argumentos del evento.</param>
        private void OperacionUnitariaClick(object sender, EventArgs e)
        {
            OperacionBinariaClick(sender, e);
            textBox1.Text = numeroAnterior;
            BotonIgual.PerformClick();
        }

        /// <summary>
        /// Método que se ejecuta cada vez que el usuario presiona el boton de algun digito.
        /// Ejemplo: 0, 1, 2, ..., 9
        /// </summary>
        /// <param name="sender">Boton que ha presionado el usuario.</param>
        /// <param name="e">Argumentos del evento.</param>
        private void DigitoClick(object sender, EventArgs e)
        {
            var boton = sender as Button;
            textBox1.Text = textBox1.Text + boton.Text;

            if (textBox1.Text.StartsWith("0"))
            {
                textBox1.Text = textBox1.Text.TrimStart('0');
            }

        }

        /// <summary>
        /// Método que se ejecuta cuando el usuario presiona el boton igual
        /// </summary>
        /// <param name="sender">Boton que ha presionado el usuario.</param>
        /// <param name="e">Argumentos del evento.</param>
        private void BotonIgual_Click(object sender, EventArgs e)
        {

            string numeroActual = textBox1.Text;

            if (!string.IsNullOrEmpty(numeroAnterior))
            {
                var operacion = ObtenerOperacion(ultimaOperacion);
                var numeroA = double.Parse(numeroAnterior);
                var numeroB = double.Parse(numeroActual);
                var resultado = operacion(numeroA, numeroB);
                textBox1.Text = resultado.ToString();
            }
        }
        
        /// <summary>
        /// Método que se ejecuta cuando el usuario presiona el boton de cambio de signo.
        /// </summary>
        /// <param name="sender">Boton que ha presionado el usuario.</param>
        /// <param name="e">Argumentos del evento.</param>
        private void BotonSigno_Click(object sender, EventArgs e)
        {
            if (textBox1.Text.StartsWith("-"))
            {
                textBox1.Text = textBox1.Text.Remove(0);
            }
            else
            {
                textBox1.Text = "-" + textBox1.Text;
            }
        }

        /// <summary>
        /// Método que se ejecuta cuando el usuario presiona el boton para colocar un punto decimal.
        /// </summary>
        /// <param name="sender">Boton que ha presionado el usuario.</param>
        /// <param name="e">Argumentos del evento.</param>
        private void BotonPuntoDecimal_Click(object sender, EventArgs e)
        {
            if (ValidarSiEsNumero(textBox1.Text) && !textBox1.Text.Contains('.'))
            {
                textBox1.Text = textBox1.Text + ".";
            }
        }

        /// <summary>
        /// Método que se ejecuta cuando el usuario presiona el boton de limpiar todas las operaciones.
        /// </summary>
        /// <param name="sender">Boton que ha presionado el usuario.</param>
        /// <param name="e">Argumentos del evento.</param>
        private void BotonLimpiarTodo_Click(object sender, EventArgs e)
        {
            numeroAnterior = "";
            ultimaOperacion = "";
            textBox1.Text = "";
        }

        /// <summary>
        /// Método que se ejecuta cuando el usuario presiona el boton de limpiar el numero que se encuentra en pantalla.
        /// </summary>
        /// <param name="sender">Boton que ha presionado el usuario.</param>
        /// <param name="e">Argumentos del evento.</param>
        private void BotonLimparElemento_Click(object sender, EventArgs e)
        {
            textBox1.Text = "";
        }

        /// <summary>
        /// Método que se ejecuta cuando el usuario presiona el boton de retroceder para borrar un numero.
        /// </summary>
        /// <param name="sender">Boton que ha presionado el usuario.</param>
        /// <param name="e">Argumentos del evento.</param>
        private void BotonRetroceder_Click(object sender, EventArgs e)
        {
            if (textBox1.Text.Length > 0)
            {
                textBox1.Text = textBox1.Text.Remove(textBox1.Text.Length - 1);
            }
        }




    }
}
